/// <reference types="graceful-fs" />
import { Token, html, DefaultTreeAdapterMap } from "parse5";
import { ParcelConfig } from "single-spa";
import { ServerResponse } from "node:http";
import { Readable } from "node:stream";
declare const nodeNames: {
    readonly APPLICATION: "application";
    readonly ASSETS: "assets";
    readonly COMMENT: "#comment";
    readonly DATA: "data";
    readonly DOCUMENT_TYPE: "#documentType";
    readonly ELEMENT: "element";
    readonly FRAGMENT: "fragment";
    readonly NODE: "node";
    readonly REDIRECT: "redirect";
    readonly ROUTE: "route";
    readonly ROUTER: "single-spa-router";
    readonly ROUTER_CONTENT: "ssl-router-content";
    readonly TEXT: "#text";
};
type SslCommentNode = DefaultTreeAdapterMap["commentNode"];
type SslDocument = DefaultTreeAdapterMap["document"];
type SslElement = DefaultTreeAdapterMap["element"];
type SslTextNode = DefaultTreeAdapterMap["textNode"];
interface SslApplication extends SslElement {
    nodeName: typeof nodeNames.APPLICATION;
    tagName: typeof nodeNames.APPLICATION;
}
interface SslRoute extends SslElement {
    nodeName: typeof nodeNames.ROUTE;
    tagName: typeof nodeNames.ROUTE;
}
type ActiveWhen = (location: Location | URL) => boolean;
interface ResolvedApplication extends SslApplication {
    _kind: typeof nodeNames.APPLICATION;
    attrs: [
    ];
    childNodes: [
    ];
    error?: string | ParcelConfig;
    loader?: string | ParcelConfig;
    name: string;
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
    props?: Record<string, unknown>;
}
interface ResolvedComment extends SslCommentNode {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.COMMENT;
}
interface ResolvedElement extends SslElement {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.ELEMENT;
    childNodes: ResolvedChild[];
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
}
interface ResolvedNode extends SslElement {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.NODE;
    attrs: [
    ];
    childNodes: ResolvedChild[];
    namespaceURI: typeof html.NS.HTML;
    node: Node;
    nodeName: typeof nodeNames.NODE;
    parentNode: null;
}
interface ResolvedRoute extends SslRoute {
    _connectedNode?: never;
    _kind: typeof nodeNames.ROUTE;
    activeWhen: ActiveWhen;
    attrs: [
    ];
    childNodes: ResolvedChild[];
    default?: boolean;
    exact?: boolean;
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
    path?: string;
    props: Record<string, unknown>;
}
interface ResolvedText extends SslTextNode {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.TEXT;
    parentNode: null;
}
type ResolvedChild = ResolvedApplication | ResolvedComment | ResolvedElement | ResolvedNode | ResolvedRoute | ResolvedText;
type ContainerEl = string | HTMLElement | SslElement;
type Redirects = Record<string, string>;
type RouteMode = "history" | "hash";
interface ResolvedRoutesConfig {
    base: string;
    childNodes: ResolvedChild[];
    containerEl: ContainerEl;
    mode?: RouteMode;
    redirects?: Redirects;
}
interface InputApplication {
    error?: string | ParcelConfig;
    loader?: string | ParcelConfig;
    name: string;
    nodeName: typeof nodeNames.APPLICATION;
    props?: Record<string, unknown>;
}
interface InputComment {
    nodeName: typeof nodeNames.COMMENT;
    data: string;
}
interface InputElement {
    attrs?: Token.Attribute[];
    childNodes?: InputRouteChild[];
    nodeName: string;
}
interface InputNode {
    node: Node;
    nodeName: typeof nodeNames.NODE;
    childNodes?: InputRouteChild[];
}
interface InputRoute {
    childNodes: InputRouteChild[];
    default?: boolean;
    exact?: boolean;
    nodeName: typeof nodeNames.ROUTE;
    path?: string;
    props: Record<string, unknown>;
}
interface InputText {
    nodeName: typeof nodeNames.TEXT;
    value: string;
}
type InputRouteChild = InputApplication | InputComment | InputElement | InputNode | InputRoute | InputText;
interface InputRoutesConfig extends Partial<Omit<ResolvedRoutesConfig, "childNodes">> {
    childNodes: InputRouteChild[];
    disableWarnings?: boolean;
}
interface HTMLLayoutData {
    errors?: Record<string, string | ParcelConfig>;
    loaders?: Record<string, any>;
    props?: Record<string, any>;
}
declare function constructRoutes(htmlOrElement: string | HTMLElement | SslElement, layoutData?: HTMLLayoutData): ResolvedRoutesConfig;
declare function constructRoutes(configObject: InputRoutesConfig): ResolvedRoutesConfig;
interface ServerLayout {
    parsedDocument: SslDocument;
    resolvedConfig: ResolvedRoutesConfig;
}
type HTMLTemplateOptions = {
    filePath: string;
} | {
    html: string;
};
declare const constructServerLayout: (templateOptions: HTMLTemplateOptions) => ServerLayout;
interface Pipeable {
    pipe: <Writable extends NodeJS.WritableStream>(stream: Writable) => Writable;
}
type StreamValue = string | Readable | Pipeable;
type StreamInput = StreamValue | PromiseLike<StreamValue>;
interface AppToRender {
    appName: string;
    propsPromise: Promise<Record<string, unknown>>;
}
interface AppHeaders {
    appHeaders: Record<string, string>;
    appProps: Record<string, unknown>;
}
interface RenderResult {
    assets?: StreamInput;
    content: StreamInput;
    props?: {} | Promise<{}>;
}
interface RenderOptions {
    assembleFinalHeaders: (appHeaders: AppHeaders[]) => Record<string, string>;
    renderApplication: (appToRender: AppToRender) => RenderResult;
    renderFragment: (name: string) => StreamValue | Promise<StreamValue>;
    res: ServerResponse;
    retrieveApplicationHeaders: (appToRender: AppToRender) => Promise<Record<string, string>>;
    retrieveProp: (name: string) => unknown | Promise<unknown>;
    serverLayout: ServerLayout;
    urlPath: string;
}
declare const sendLayoutHTTPResponse: (renderOptions: RenderOptions) => Promise<void>;
export { constructRoutes, HTMLLayoutData, InputApplication, InputComment, InputElement, InputNode, InputRoute, InputRouteChild, InputRoutesConfig, InputText, ResolvedRoutesConfig, constructServerLayout, HTMLTemplateOptions, ServerLayout, AppHeaders, AppToRender, StreamValue, RenderOptions, RenderResult, sendLayoutHTTPResponse };
