import { Token, html, DefaultTreeAdapterMap } from "parse5";
import { ParcelConfig, RegisterApplicationConfig, LifeCycles, SingleSpaCustomEventDetail } from "single-spa";
declare const nodeNames: {
    readonly APPLICATION: "application";
    readonly ASSETS: "assets";
    readonly COMMENT: "#comment";
    readonly DATA: "data";
    readonly DOCUMENT_TYPE: "#documentType";
    readonly ELEMENT: "element";
    readonly FRAGMENT: "fragment";
    readonly NODE: "node";
    readonly REDIRECT: "redirect";
    readonly ROUTE: "route";
    readonly ROUTER: "single-spa-router";
    readonly ROUTER_CONTENT: "ssl-router-content";
    readonly TEXT: "#text";
};
type SslCommentNode = DefaultTreeAdapterMap["commentNode"];
type SslElement = DefaultTreeAdapterMap["element"];
type SslTextNode = DefaultTreeAdapterMap["textNode"];
interface SslApplication extends SslElement {
    nodeName: typeof nodeNames.APPLICATION;
    tagName: typeof nodeNames.APPLICATION;
}
interface SslRoute extends SslElement {
    nodeName: typeof nodeNames.ROUTE;
    tagName: typeof nodeNames.ROUTE;
}
type ActiveWhen = (location: Location | URL) => boolean;
interface ResolvedApplication extends SslApplication {
    _kind: typeof nodeNames.APPLICATION;
    attrs: [
    ];
    childNodes: [
    ];
    error?: string | ParcelConfig;
    loader?: string | ParcelConfig;
    name: string;
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
    props?: Record<string, unknown>;
}
interface ResolvedComment extends SslCommentNode {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.COMMENT;
}
interface ResolvedElement extends SslElement {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.ELEMENT;
    childNodes: ResolvedChild[];
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
}
interface ResolvedNode extends SslElement {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.NODE;
    attrs: [
    ];
    childNodes: ResolvedChild[];
    namespaceURI: typeof html.NS.HTML;
    node: Node;
    nodeName: typeof nodeNames.NODE;
    parentNode: null;
}
interface ResolvedRoute extends SslRoute {
    _connectedNode?: never;
    _kind: typeof nodeNames.ROUTE;
    activeWhen: ActiveWhen;
    attrs: [
    ];
    childNodes: ResolvedChild[];
    default?: boolean;
    exact?: boolean;
    namespaceURI: typeof html.NS.HTML;
    parentNode: null;
    path?: string;
    props: Record<string, unknown>;
}
interface ResolvedText extends SslTextNode {
    _connectedNode: Nullable<Node>;
    _kind: typeof nodeNames.TEXT;
    parentNode: null;
}
type ResolvedChild = ResolvedApplication | ResolvedComment | ResolvedElement | ResolvedNode | ResolvedRoute | ResolvedText;
type ContainerEl = string | HTMLElement | SslElement;
type Redirects = Record<string, string>;
type RouteMode = "history" | "hash";
interface ResolvedRoutesConfig {
    base: string;
    childNodes: ResolvedChild[];
    containerEl: ContainerEl;
    mode?: RouteMode;
    redirects?: Redirects;
}
interface InputApplication {
    error?: string | ParcelConfig;
    loader?: string | ParcelConfig;
    name: string;
    nodeName: typeof nodeNames.APPLICATION;
    props?: Record<string, unknown>;
}
interface InputComment {
    nodeName: typeof nodeNames.COMMENT;
    data: string;
}
interface InputElement {
    attrs?: Token.Attribute[];
    childNodes?: InputRouteChild[];
    nodeName: string;
}
interface InputNode {
    node: Node;
    nodeName: typeof nodeNames.NODE;
    childNodes?: InputRouteChild[];
}
interface InputRoute {
    childNodes: InputRouteChild[];
    default?: boolean;
    exact?: boolean;
    nodeName: typeof nodeNames.ROUTE;
    path?: string;
    props: Record<string, unknown>;
}
interface InputText {
    nodeName: typeof nodeNames.TEXT;
    value: string;
}
type InputRouteChild = InputApplication | InputComment | InputElement | InputNode | InputRoute | InputText;
interface InputRoutesConfig extends Partial<Omit<ResolvedRoutesConfig, "childNodes">> {
    childNodes: InputRouteChild[];
    disableWarnings?: boolean;
}
interface HTMLLayoutData {
    errors?: Record<string, string | ParcelConfig>;
    loaders?: Record<string, any>;
    props?: Record<string, any>;
}
declare function constructRoutes(htmlOrElement: string | HTMLElement | SslElement, layoutData?: HTMLLayoutData): ResolvedRoutesConfig;
declare function constructRoutes(configObject: InputRoutesConfig): ResolvedRoutesConfig;
declare const applicationElementId: (fullName: string) => string;
declare const getAppName: (fullName: string) => string;
declare const getAppProps: <TProps>(fullName: string) => TProps | undefined;
declare const getLayoutConfig: () => ResolvedRoutesConfig | undefined;
interface LoadAppProps {
    name: string;
}
interface ApplicationOptions {
    config: ResolvedRoutesConfig;
    loadApp: (config: LoadAppProps) => Promise<LifeCycles>;
}
declare const constructApplications: ({ config: { childNodes }, loadApp }: ApplicationOptions) => RegisterApplicationConfig[];
interface LayoutEngine {
    activate: () => void;
    deactivate: () => void;
    isActive: () => boolean;
}
interface LayoutEngineOptions {
    active?: boolean;
    config: ResolvedRoutesConfig;
}
type SingleSpaEventListener = (ev: CustomEvent<SingleSpaCustomEventDetail>) => void;
type SingleSpaEvent = "single-spa:app-change" | "single-spa:before-app-change" | "single-spa:before-first-mount" | "single-spa:before-mount-routing-event" | "single-spa:before-no-app-change" | "single-spa:before-routing-event" | "single-spa:first-mount" | "single-spa:no-app-change" | "single-spa:routing-event";
declare global {
    interface Window {
        addEventListener(type: SingleSpaEvent, listener: SingleSpaEventListener): void;
        removeEventListener(type: SingleSpaEvent, listener: SingleSpaEventListener): void;
    }
}
declare const constructLayoutEngine: ({ config, active }: LayoutEngineOptions) => LayoutEngine;
export { constructRoutes, HTMLLayoutData, InputApplication, InputComment, InputElement, InputNode, InputRoute, InputRouteChild, InputRoutesConfig, InputText, ResolvedRoutesConfig, applicationElementId, getAppName, getAppProps, getLayoutConfig, constructApplications, ApplicationOptions, constructLayoutEngine, LayoutEngine, LayoutEngineOptions };
